/*
 * Copyright (C) 2004 Zaheer Abbas Merali <zaheerabbas at merali.org>
 *                    Mahmood Rehemtulla  <mhr at blueyonder.co.uk>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef _GSTSAVE_GUI_H
#define _GSTSAVE_GUI_H

#include <gtk/gtk.h>
#include <glib.h>
#include <gst/gst.h>

#include <glade/glade.h>

struct pipeline_elements;
struct stream_details;


/* GUI elements */
struct gstsave_gui
{
  GHashTable* table;
  GList* relay_checkbuttons;
  
  GladeXML* windowdefs;
  struct stream_details* sd;
  gboolean realised;

#ifdef GNOME
  GList* recording_gm_profile_radiobuttons;
#endif
};

/* function prototypes */
/* gui callbacks */

static gboolean acast_main_window_closed (GtkWidget* w, GdkEvent* e, gpointer user_data);
static gboolean acast_preferences_window_closed (GtkWidget* w, GdkEvent* e, gpointer user_data);
static gboolean checkbox_toggled(GtkToggleButton* tb, gpointer user_data);
static gboolean changefilename_button_clicked(GtkButton* b, gpointer user_data);
static gboolean exit_button_clicked(GtkWidget* w, gpointer user_data);
static gboolean listen_checkbox_toggled(GtkToggleButton* tb, gpointer user_data);
static gboolean execute_button_clicked(GtkButton* b, struct gstsave_gui* gsgui);
static void save_preferences_as (GtkWidget* w, gpointer user_data);
static void save_preferences(GtkWidget* w, gpointer user_data);
static void open_file (GtkWidget* w, struct gstsave_gui* gsgui);
static gboolean gmp_radiobutton_toggled (GtkWidget* blah, struct gstsave_gui* gsgui);
static gboolean make_apply_button_sensitive(GtkWidget* blah, struct gstsave_gui* gsgui);
static gboolean make_icecastsend_sensitive(GtkWidget* blah, struct gstsave_gui* gsgui);
static gboolean make_icecastsend_unsensitive(GtkWidget* blah, struct gstsave_gui* gsgui);
static gboolean change_spinbutton_of_scale(GtkHScale* scale, GtkSpinButton* spin);
static gboolean change_scale_of_spinbutton(GtkSpinButton* spin, GtkHScale* scale);
void remove_from_container (GtkWidget* widget, gpointer user_data);
static gboolean close_preferences_window(GtkWidget* blah, struct gstsave_gui* gsgui);
static gboolean show_preferences_window(GtkWidget* blah, struct gstsave_gui* gsgui);
void advanced_view_clicked (GtkExpander* widget, GParamSpec* blah, 
			    struct gstsave_gui* gsgui);
/* gui setup */
//void save_preferences(struct gstsave_gui* gsgui);

void update_stream_checkboxes(struct gstsave_gui* gsgui);

void setup_preferences_window(struct gstsave_gui* gsgui);
void setup_gui (struct gstsave_gui* gsgui);
void apply_config_changes (struct gstsave_gui* gsgui);

/* gst related */
#ifdef GST_06
static void xvideosink_have_size(GObject* object, gint arg0, gint arg1, gpointer user_data);
static void xvideosink_have_xid(GObject* object, gint arg0, gpointer user_data);
#else
void xvimagesink_have_size(GObject* object, gint arg0, gint arg1, gpointer user_data);
gboolean embed_xvideosink(struct gstsave_gui* gsgui);
static void drawingarea_after_realize(GObject* object,gpointer user_data);
#endif
static void pipeline_changed (GstElement* object, gint arg0, gint arg1, gpointer user_data);

/* relays */
static void gui_add_relay(gconstpointer key, gconstpointer value, gpointer user_data);
static gboolean save_relay_preferences(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter, gpointer user_data);
static void add_relay_to_list_view(gconstpointer name, gconstpointer value, gpointer user_data);
static void relay_treeview_cursor_changed(GtkTreeView* view, gpointer data);
void relay_delete_button_clicked(GtkWidget* widget, gpointer user_data);
void relay_add_button_clicked(GtkWidget* widget, gpointer user_data);
static void relay_checkbutton_toggled(GtkCheckButton* button, gpointer user_data);
static void update_relay_checkbox(GtkWidget* checkbox, struct gstsave_gui* gsgui);


#endif
